/*
 * This file is part of ViaVersion - https://github.com/ViaVersion/ViaVersion
 * Copyright (C) 2016-2024 ViaVersion and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.viaversion.viaversion.api.minecraft.item.data;

import com.viaversion.viaversion.api.type.Type;
import io.netty.buffer.ByteBuf;
import java.util.UUID;

public final class ModifierData {

    public static final Type<ModifierData> TYPE = new Type<ModifierData>(ModifierData.class) {
        @Override
        public ModifierData read(final ByteBuf buffer) throws Exception {
            final UUID uuid = Type.UUID.read(buffer);
            final String name = Type.STRING.read(buffer);
            final double amount = buffer.readDouble();
            final int operation = Type.VAR_INT.readPrimitive(buffer);
            return new ModifierData(uuid, name, amount, operation);
        }

        @Override
        public void write(final ByteBuf buffer, final ModifierData value) throws Exception {
            Type.UUID.write(buffer, value.uuid);
            Type.STRING.write(buffer, value.name);
            buffer.writeDouble(value.amount);
            Type.VAR_INT.writePrimitive(buffer, value.operation);
        }
    };

    private final UUID uuid;
    private final String name;
    private final double amount;
    private final int operation;

    public ModifierData(final UUID uuid, final String name, final double amount, final int operation) {
        this.uuid = uuid;
        this.name = name;
        this.amount = amount;
        this.operation = operation;
    }

    public UUID uuid() {
        return uuid;
    }

    public String name() {
        return name;
    }

    public double amount() {
        return amount;
    }

    public int operation() {
        return operation;
    }
}
