/*
 * This file is part of ViaVersion - https://github.com/ViaVersion/ViaVersion
 * Copyright (C) 2016-2024 ViaVersion and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.viaversion.viaversion.api.protocol.packet.provider;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import com.viaversion.viaversion.api.protocol.packet.PacketType;
import org.checkerframework.checker.nullness.qual.Nullable;

final class PacketTypeArrayMap<P extends PacketType> implements PacketTypeMap<P> {
    private final Map<String, P> packetsByName;
    private final P[] packets;

    PacketTypeArrayMap(Map<String, P> packetsByName, P[] packets) {
        this.packetsByName = packetsByName;
        this.packets = packets;
    }

    @Override
    public @Nullable P typeByName(String packetTypeName) {
        return packetsByName.get(packetTypeName);
    }

    @Override
    public @Nullable P typeById(int packetTypeId) {
        return packetTypeId >= 0 && packetTypeId < packets.length ? packets[packetTypeId] : null;
    }

    @Override
    public Collection<P> types() {
        return Arrays.asList(packets);
    }
}