/*
 * This file is part of ViaVersion - https://github.com/ViaVersion/ViaVersion
 * Copyright (C) 2016-2024 ViaVersion and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.viaversion.viaversion.api.protocol.version;

import java.util.Set;

public interface BlockedProtocolVersions {

    /**
     * Returns whether the given protocol version is blocked per boundary ranges or individually blocked versions.
     *
     * @param protocolVersion protocol version
     * @return whether the given protocol version is blocked
     */
    boolean contains(ProtocolVersion protocolVersion);

    /**
     * Returns the boundary below which protocol versions are blocked, or -1 if none is set.
     *
     * @return exclusive boundary below which protocol versions are blocked, or -1 if none
     */
    ProtocolVersion blocksBelow();

    /**
     * Returns the boundary above which protocol versions are blocked, or -1 if none is set.
     *
     * @return exclusive boundary above which protocol versions are blocked, or -1 if none
     */
    ProtocolVersion blocksAbove();

    /**
     * Returns a set of blocked protocol versions between the outer block ranges.
     *
     * @return set of blocked protocol versions between the outer block ranges
     */
    Set<ProtocolVersion> singleBlockedVersions();
}
